import { Locator, Page, expect, test } from '@playwright/test'

import {
  checkAccessibility,
  clickElement,
  clickElementByTestId,
  locateElementBySelector,
  locateElementByTestId,
  waitForElementByTestId,
} from './helpers'

test.beforeEach(async ({ page }) => {
  await page.goto('/apps/donations')
})

const filterForLocation = async (
  page: Page,
  location: string,
  { assertAccessibility = false, useSuggestion = false } = {}
) => {
  // open search modal
  await clickElementByTestId(page, 'donations-search-button')

  if (assertAccessibility) {
    await checkAccessibility(page)
  }

  // enter location
  const locationInput = await locateElementByTestId(page, 'donations-search-location-input')
  await locationInput.fill(location)

  // expect suggestions
  const element = await locateElementBySelector(page, '[role="menuitem"]')
  if (useSuggestion) {
    await clickElement(element)
  }

  // close search modal
  await clickElementByTestId(page, 'donations-search-modal-close-btn')

  // wait for results
  await waitForElementByTestId(page, 'donations-list-container')
}

const extractResultCount = async (element: Locator): Promise<number> => {
  const textContent = await element.textContent()
  const numberMatch = textContent?.match(/\d+/)
  if (numberMatch && numberMatch.length) {
    return parseInt(numberMatch[0], 10)
  }
  return 0
}

const extractNewsCount = async (element: Locator): Promise<number | undefined> => {
  const textContent = await element.textContent()
  const numberNews = textContent?.match(/\(([^)]+)\)/)
  if (numberNews && numberNews.length) {
    return parseInt(numberNews[1], 10)
  }
  return undefined
}

const resetFilterInModal = async (page: Page) => {
  // open search modal
  await clickElementByTestId(page, 'donations-search-button')

  // reset filter
  await clickElementByTestId(page, 'donations-search-form-reset')

  // close search modal
  await clickElementByTestId(page, 'donations-search-modal-close-btn')

  // wait for results
  await waitForElementByTestId(page, 'donations-list-container')
}

// sometimes responses to Betterplace take a little longer (10s < response latency < 30s)
// increase timeout from default of 5 seconds to 30 seconds
const betterplaceTimeout = { timeout: 30_000 }

test.describe('@Donations', () => {
  test('should be accessible from home screen', async ({ page }) => {
    await page.goto('/')
    await clickElementByTestId(page, 'home-apps-button')
    await clickElementByTestId(page, 'app-tile-donations')

    await waitForElementByTestId(page, 'donations-list-container')
    expect(page.url()).toContain('/apps/donations')
  })

  test('should display multiple projects', async ({ page }) => {
    await waitForElementByTestId(page, 'donations-list-container')
    const listItems = await locateElementByTestId(page, 'donations-list-item')
    const itemCount = await listItems.count()
    expect(itemCount).toBeGreaterThan(1)

    await checkAccessibility(page)
  })

  test('should allow pagination', async ({ page }) => {
    await waitForElementByTestId(page, 'donations-list-container')
    const countListItems = async () => {
      const listItems = await locateElementByTestId(page, 'donations-list-item')
      return await listItems.count()
    }
    await expect.poll(countListItems).toBeGreaterThan(0)
    const listItemCountBefore = await countListItems()
    await clickElementByTestId(page, 'btn-donation-list-load-more')
    await expect.poll(countListItems, betterplaceTimeout).toBeGreaterThan(listItemCountBefore)
  })

  test('should display project details', async ({ page, context }) => {
    await waitForElementByTestId(page, 'donations-list-container')
    // click first element in list
    await clickElementByTestId(page, 'donations-list-item')

    await waitForElementByTestId(page, 'donations-project-detail-heading')

    await checkAccessibility(page)

    const projectUrlPrefix = '/apps/donations/project/'
    const projectUrl = page.url()
    const projectId = projectUrl.substring(projectUrl.indexOf(projectUrlPrefix) + projectUrlPrefix.length)
    expect(projectUrl).toContain(projectUrlPrefix)

    // check if clicking on donate button actually opens a tab with the correct URL
    await waitForElementByTestId(page, 'donations-project-detail-donate-btn')
    // follows https://playwright.dev/docs/pages#handling-new-pages
    const [donationPage] = await Promise.all([
      context.waitForEvent('page'),
      clickElementByTestId(page, 'donations-project-detail-donate-btn'), // opens a new tab
    ])
    const donationUrl = donationPage.url()
    expect(donationUrl).toMatch(/https:\/\/www.betterplace.org\/.*\/donate\/platform\/projects\/.*/)
    const donationUrlParts = donationUrl.split('/')
    expect(donationUrlParts[donationUrlParts.length - 1]).toBe(projectId)
    await donationPage.close() // close newly opened tab

    // check if clicking on initiative button actually opens a tab with a reasonable URL
    await waitForElementByTestId(page, 'donations-project-initiative-btn')
    const [initiativePage] = await Promise.all([
      context.waitForEvent('page'),
      clickElementByTestId(page, 'donations-project-initiative-btn'), // opens a new tab
    ])
    const initiativeUrl = initiativePage.url()
    expect(initiativeUrl).toMatch(/https:\/\/www.betterplace.org\/.*\/organisations\/.*/)
    await initiativePage.close() // close newly opened tab
  })

  test('should display correct news count on project details', async ({ page }) => {
    const countSpotlightNews = async () => await (await locateElementByTestId(page, 'donation-news-spotlight')).count()
    const countOtherNews = async () => await (await page.locator('data-testid=donations-news-other-item')).count()

    await waitForElementByTestId(page, 'donations-list-container')

    // open first donation opportunity
    await clickElementByTestId(page, 'donations-list-item')
    await waitForElementByTestId(page, 'donations-project-detail-heading')

    const hasNews = await page.$('data-testid=donations-news-count')
    test.skip(!hasNews, 'donation opportunity has no news, skipping assertions')

    const newsCount = await extractNewsCount(await locateElementByTestId(page, 'donations-news-count'))
    // spotlight is shown when any news exist
    const expectedSpotlightCount = newsCount === 0 ? 0 : 1
    // up to 4 other news are shown after the spotlight
    const expectedOtherCount = !newsCount || newsCount === 0 ? 0 : newsCount >= 5 ? 4 : newsCount - 1

    expect(newsCount).toBeDefined()
    await expect.poll(countSpotlightNews, { timeout: 10000 }).toBe(expectedSpotlightCount)
    await expect.poll(countOtherNews, { timeout: 10000 }).toBe(expectedOtherCount)
  })

  test('should filter donations by location and reset', async ({ page }) => {
    // filter for "Germany"
    await filterForLocation(page, 'Germany', { assertAccessibility: true })

    // assert results
    let heading = await locateElementByTestId(page, 'donations-list-search-heading')
    await expect(heading).toContainText('Germany')
    let summary = await locateElementByTestId(page, 'donations-list-search-summary')
    const resultCountGermany = await extractResultCount(summary)
    expect(resultCountGermany).toBeGreaterThan(0)

    await checkAccessibility(page)

    // reset filter in list
    await clickElementByTestId(page, 'donations-list-search-reset')

    // assert reset
    await waitForElementByTestId(page, 'donations-list-unfiltered')

    // filter for "Hamburg"
    await filterForLocation(page, 'Hamburg', { useSuggestion: true })

    // assert and compare results
    heading = await locateElementByTestId(page, 'donations-list-search-heading')
    await expect(heading).toContainText('Hamburg')
    summary = await locateElementByTestId(page, 'donations-list-search-summary')
    const resultCountHamburg = await extractResultCount(summary)
    expect(resultCountHamburg).toBeGreaterThan(0)
    expect(resultCountHamburg).toBeLessThan(resultCountGermany)

    // reset filter in modal
    await resetFilterInModal(page)

    // assert reset
    await waitForElementByTestId(page, 'donations-list-unfiltered')
  })

  test('should display details of filter results', async ({ page }) => {
    // filter for "Hamburg"
    await filterForLocation(page, 'Hamburg')

    // expect filtered list
    let heading = await locateElementByTestId(page, 'donations-list-search-heading')
    await expect(heading).toContainText('Hamburg')

    // click first element in list
    await clickElementByTestId(page, 'donations-list-item')

    // expect detail view
    await waitForElementByTestId(page, 'donations-project-detail-heading')

    // navigate back
    await clickElementByTestId(page, 'nav-button-back')

    // expect filtered list
    heading = await locateElementByTestId(page, 'donations-list-search-heading')
    await expect(heading).toContainText('Hamburg')
  })
})
